import makeRequest from "@root/api/makeRequest";

import { parseAxiosError } from "@root/utils/parse-error";

import type { Discount } from "@frontend/kitui";
import type { CreateDiscountBody, DiscountType, GetDiscountResponse } from "@root/model/discount";

const API_URL = `${process.env.REACT_APP_DOMAIN}/price`;

interface CreateDiscountParams {
	purchasesAmount: number;
	cartPurchasesAmount: number;
	discountMinValue: number;
	discountFactor: number;
	discountDescription: string;
	discountName: string;
	/** ISO string */
	startDate: string;
	/** ISO string */
	endDate: string;
	serviceType: string;
	discountType: DiscountType;
	privilegeId: string;
}

export function createDiscountObject({
	endDate,
	startDate,
	discountName,
	cartPurchasesAmount,
	discountDescription,
	discountFactor,
	discountMinValue,
	purchasesAmount,
	serviceType,
	discountType,
	privilegeId,
}: CreateDiscountParams) {
	const discount: CreateDiscountBody = {
		Name: discountName,
		Layer: 1,
		Description: discountDescription,
		Condition: {
			Period: {
				From: startDate,
				To: endDate,
			},
			User: "",
			UserType: "",
			Coupon: "",
			Usage: 0,
			PurchasesAmount: 0,
			CartPurchasesAmount: 0,
			Product: "",
			Term: 0,
			PriceFrom: 0,
			Group: "",
		},
		Target: {
			Factor: discountFactor,
			TargetScope: "Sum",
			Overhelm: false,
			TargetGroup: "",
			Products: [
				{
					ID: "",
					Factor: 0,
					Overhelm: false,
				},
			],
		},
	};

	switch (discountType) {
		case "privilege":
			discount.Layer = 1;
			discount.Condition.Product = privilegeId;
			discount.Condition.Term = discountMinValue / 100;
			discount.Target.Products = [
				{
					Factor: discountFactor,
					ID: privilegeId,
					Overhelm: false,
				},
			];
			break;
		case "service":
			discount.Layer = 2;
			discount.Condition.PriceFrom = discountMinValue;
			discount.Condition.Group = serviceType;
			discount.Target.TargetGroup = serviceType;
			break;
		case "cartPurchasesAmount":
			discount.Layer = 3;
			discount.Condition.CartPurchasesAmount = cartPurchasesAmount;
			break;
		case "purchasesAmount":
			discount.Layer = 4;
			discount.Condition.PurchasesAmount = purchasesAmount;
			break;
	}

	return discount;
}

export const changeDiscount = async (discountId: string, discount: Discount): Promise<[unknown, string?]> => {
	try {
		const changeDiscountResponse = await makeRequest<Discount, unknown>({
			method: "PATCH",
			url: `${API_URL}/discount/${discountId}`,
			body: discount,
			useToken: true,
		});

		return [changeDiscountResponse];
	} catch (nativeError) {
		const [error] = parseAxiosError(nativeError);

		return [null, `Ошибка изменения скидки. ${error}`];
	}
};

export const createDiscount = async (discountParams: CreateDiscountParams): Promise<[Discount | null, string?]> => {
	const discount = createDiscountObject(discountParams);

	try {
		const createdDiscountResponse = await makeRequest<CreateDiscountBody, Discount>({
			method: "POST",
			url: `${API_URL}/discount`,
			body: discount,
			useToken: true,
		});

		return [createdDiscountResponse];
	} catch (nativeError) {
		const [error] = parseAxiosError(nativeError);

		return [null, `Ошибка создания скидки. ${error}`];
	}
};

export const deleteDiscount = async (discountId: string): Promise<[Discount | null, string?]> => {
	try {
		const deleteDiscountResponse = await makeRequest<never, Discount>({
			method: "DELETE",
			url: `${API_URL}/discount/${discountId}`,
			useToken: true,
		});

		return [deleteDiscountResponse];
	} catch (nativeError) {
		const [error] = parseAxiosError(nativeError);

		return [null, `Ошибка удаления скидки. ${error}`];
	}
};

export const patchDiscount = async (
	discountId: string,
	discountParams: CreateDiscountParams
): Promise<[Discount | null, string?]> => {
	const discount = createDiscountObject(discountParams);

	try {
		const patchDiscountResponse = await makeRequest<CreateDiscountBody, Discount>({
			method: "PATCH",
			url: `${API_URL}/discount/${discountId}`,
			body: discount,
			useToken: true,
		});

		return [patchDiscountResponse];
	} catch (nativeError) {
		const [error] = parseAxiosError(nativeError);

		return [null, `Ошибка изменения скидки. ${error}`];
	}
};

export const requestDiscounts = async (): Promise<[GetDiscountResponse | null, string?]> => {
	try {
		const discountsResponse = await makeRequest<never, GetDiscountResponse>({
			method: "GET",
			url: `${API_URL}/discounts`,
			useToken: true,
		});

		return [discountsResponse];
	} catch (nativeError) {
		const [error] = parseAxiosError(nativeError);

		return [null, `Ошибка получения скидок. ${error}`];
	}
};

export const getDiscounts = async (): Promise<[Discount[] | null, string?]> => {
	try {
		const discountsResponse = await makeRequest<never, GetDiscountResponse>({
			method: "GET",
			url: `${API_URL}/discounts`,
			useToken: true,
		});

		return [discountsResponse.Discounts.filter((discount) => !discount.Deprecated)];
	} catch (nativeError) {
		const [error] = parseAxiosError(nativeError);

		return [null, `Ошибка получения списка скидок. ${error}`];
	}
};
