import { useCallback, useRef } from "react";
import useSwr, { mutate } from "swr";
import { enqueueSnackbar } from "notistack";
import { promocodeApi } from "./requests";

import type { CreatePromocodeBody, EditPromocodeBody, PromocodeList } from "@root/model/promocodes";

export function usePromocodes(
	page: number,
	pageSize: number,
	promocodeId: string,
	to: number,
	from: number,
	active: boolean
) {
	const promocodesCountRef = useRef<number>(0);
	const swrResponse = useSwr(
		["promocodes", page, pageSize, active],
		async (key) => {
			const [promocodeList, promocodeListError] = await promocodeApi.getPromocodeList({
				limit: key[2],
				filter: {
					active: key[3],
				},
				page: key[1],
			});

			if (promocodeListError || !promocodeList) {
				throw new Error(promocodeListError);
			}

			promocodesCountRef.current = promocodeList.count;
			return promocodeList;
		},
		{
			onError(err) {
				console.error("Error fetching promocodes", err);
				enqueueSnackbar(err.message, { variant: "error" });
			},
			focusThrottleInterval: 60e3,
			keepPreviousData: true,
		}
	);

	const createPromocode = useCallback(
		async function (body: CreatePromocodeBody) {
			const [, createError] = await promocodeApi.createPromocode(body);

			if (createError) {
				enqueueSnackbar(createError, { variant: "error" });

				return;
			}

			mutate(["promocodes", page, pageSize]);
		},
		[page, pageSize]
	);

	const editPromocode = useCallback(
		async function (body: EditPromocodeBody) {
			const [, editError] = await promocodeApi.editPromocode(body);

			if (editError) {
				enqueueSnackbar(editError, { variant: "error" });

				return;
			}

			mutate<PromocodeList | undefined, void>(["promocodes", page, pageSize, active]);
			const [, getPromocodesError] = await promocodeApi.getPromocodeList({
				limit: pageSize,
				filter: {
					active: active,
				},
				page: page,
			});

			if (getPromocodesError) {
				enqueueSnackbar(getPromocodesError, { variant: "error" });
			}
		},
		[page, pageSize]
	);

	const deletePromocode = useCallback(
		async function (id: string) {
			try {
				await mutate<PromocodeList | undefined, void>(
					["promocodes", page, pageSize],
					async () => {
						const [_, deleteError] = await promocodeApi.deletePromocode(id);

						if (deleteError) {
							throw new Error(deleteError);
						}
					},
					{
						optimisticData(currentData, displayedData) {
							if (!displayedData) return;

							return {
								count: displayedData.count - 1,
								items: displayedData.items.filter((item) => item.id !== id),
							};
						},
						rollbackOnError: true,
						populateCache(result, currentData) {
							if (!currentData) return;

							return {
								count: currentData.count - 1,
								items: currentData.items.filter((item) => item.id !== id),
							};
						},
					}
				);
			} catch (error) {
				console.error("Error deleting promocode", error);
				if (error instanceof Error) enqueueSnackbar(error.message, { variant: "error" });
			}
		},
		[page, pageSize]
	);

	const promocodeStatistics = useSwr(
		["promocodeStatistics", promocodeId, from, to],
		async ([_, id, from, to]) => {
			if (!id) {
				return null;
			}

			const [promocodeStatisticsResponse, promocodeStatisticsError] = await promocodeApi.getPromocodeStatistics(
				id,
				from,
				to
			);

			if (promocodeStatisticsError) {
				throw new Error(promocodeStatisticsError);
			}

			return promocodeStatisticsResponse;
		},
		{
			onError(err) {
				console.error("Error fetching promocode statistics", err);
				enqueueSnackbar(err.message, { variant: "error" });
			},
			focusThrottleInterval: 60e3,
			keepPreviousData: true,
		}
	);

	const createFastLink = useCallback(
		async function (id: string) {
			const [, createError] = await promocodeApi.createFastlink(id);

			if (createError) {
				enqueueSnackbar(createError, { variant: "error" });

				return;
			}

			mutate(["promocodes", page, pageSize]);
		},
		[page, pageSize]
	);

	return {
		...swrResponse,
		createPromocode,
		deletePromocode,
		editPromocode,
		createFastLink,
		promocodeStatistics: promocodeStatistics.data,
		promocodesCount: promocodesCountRef.current,
	};
}

export function useAllPromocodes() {
	const { data } = useSwr(
		["allPromocodes"],
		async () => {
			const [promocodes, promocodesError] = await promocodeApi.getAllPromocodes();

			if (promocodesError || !promocodes) {
				throw new Error(promocodesError);
			}

			return promocodes;
		},
		{
			keepPreviousData: true,
			suspense: true,
			onError(err) {
				console.error("Error fetching all promocodes", err);
				enqueueSnackbar(err.message, { variant: "error" });
			},
		}
	);

	return data;
}

export function useNotActivePromocodes() {
	const { data } = useSwr(
		"notActivePromocodes",
		async () => {
			const [promocodes, promocodesError] = await promocodeApi.getNotActivePromocodes();

			if (promocodesError || !promocodes) {
				throw new Error(promocodesError);
			}

			return promocodes;
		},
		{
			keepPreviousData: true,
			suspense: true,
			onError(err) {
				console.error("Error fetching all promocodes", err);
				enqueueSnackbar(err.message, { variant: "error" });
			},
		}
	);

	return data;
}
