import { useEffect } from "react";
import { Box, IconButton, useTheme, Tooltip } from "@mui/material";
import { DataGrid, GridColDef, GridRowsProp, GridToolbar } from "@mui/x-data-grid";
import {
	openEditDiscountDialog,
	setSelectedDiscountIds,
	updateDiscount,
	useDiscountStore,
} from "@root/stores/discounts";
import DeleteIcon from "@mui/icons-material/Delete";
import EditIcon from "@mui/icons-material/Edit";
import { deleteDiscount } from "@root/api/discounts/requests";
import { GridSelectionModel } from "@mui/x-data-grid";
import { requestDiscounts } from "@root/services/discounts.service";
import AutorenewIcon from "@mui/icons-material/Autorenew";
import { formatDiscountFactor } from "@root/utils/formatDiscountFactor";
import { mutate } from "swr";

const columns: GridColDef[] = [
	// {
	//     field: "id",
	//     headerName: "ID",
	//     width: 70,
	//     sortable: false,
	// },
	{
		field: "name",
		headerName: "Название скидки",
		width: 150,
		sortable: false,
		renderCell: ({ row, value }) => <Box color={row.deleted && "#ff4545"}>{value}</Box>,
	},
	{
		field: "description",
		headerName: "Описание",
		width: 120,
		sortable: false,
	},
	{
		field: "conditionType",
		headerName: "Тип условия",
		width: 120,
		sortable: false,
	},
	{
		field: "factor",
		headerName: "Процент скидки",
		width: 120,
		sortable: false,
	},
	{
		field: "value",
		headerName: "Значение",
		width: 120,
		sortable: false,
	},
	{
		field: "active",
		headerName: "Активна",
		width: 80,
		sortable: false,
	},
	{
		field: "edit",
		headerName: "Изменить",
		width: 80,
		sortable: false,
		renderCell: ({ row }) => {
			return (
				<IconButton
					onClick={() => {
						openEditDiscountDialog(row.id);
					}}
				>
					<EditIcon />
				</IconButton>
			);
		},
	},
	{
		field: "delete",
		headerName: "Удалить",
		width: 80,
		sortable: false,
		renderCell: ({ row }) => (
			<IconButton
				disabled={row.deleted}
				onClick={() => {
					deleteDiscount(row.id).then(([discount]) => {
						mutate("discounts");
						if (discount) {
							updateDiscount(discount);
						}
					});
				}}
			>
				<DeleteIcon />
			</IconButton>
		),
	},
];

const layerTranslate = ["", "Товар", "Сервис", "корзина", "лояльность"];
const layerValue = ["", "Term", "PriceFrom", "CartPurchasesAmount", "PurchasesAmount"];
interface Props {
	selectedRowsHC: (array: GridSelectionModel) => void;
}
export default function DiscountDataGrid({ selectedRowsHC }: Props) {
	const theme = useTheme();
	const selectedDiscountIds = useDiscountStore((state) => state.selectedDiscountIds);
	const realDiscounts = useDiscountStore((state) => state.discounts);

	useEffect(() => {
		requestDiscounts();
	}, []);

	const rowBackDicounts: GridRowsProp = realDiscounts
		.filter(({ Layer }) => Layer > 0)
		.map((discount) => {
			return {
				id: discount.ID,
				name: discount.Name,
				description: discount.Description,
				conditionType: layerTranslate[discount.Layer],
				factor: formatDiscountFactor(discount.Target.Factor),
				value:
					discount.Layer === 1
						? discount.Condition[layerValue[discount.Layer] as keyof typeof discount.Condition]
						: Number(discount.Condition[layerValue[discount.Layer] as keyof typeof discount.Condition]) / 100,
				active: discount.Deprecated ? "🚫" : "✅",
				deleted: discount.Audit.Deleted,
			};
		});

	return (
		<Box sx={{ width: "100%", marginTop: "55px", p: "16px", maxWidth: "1000px" }}>
			<Tooltip title="обновить список привилегий">
				<IconButton onClick={requestDiscounts} style={{ display: "block", margin: "0 auto" }}>
					<AutorenewIcon sx={{ color: "white" }} />
				</IconButton>
			</Tooltip>
			<Box sx={{ height: 600 }}>
				<DataGrid
					checkboxSelection={true}
					rows={rowBackDicounts}
					columns={columns}
					selectionModel={selectedDiscountIds}
					onSelectionModelChange={(array: GridSelectionModel) => {
						selectedRowsHC(array);
						setSelectedDiscountIds(array);
					}}
					disableSelectionOnClick
					sx={{
						color: theme.palette.secondary.main,
						"& .MuiDataGrid-iconSeparator": {
							display: "none",
						},
						"& .css-levciy-MuiTablePagination-displayedRows": {
							color: theme.palette.secondary.main,
						},
						"& .MuiSvgIcon-root": {
							color: theme.palette.secondary.main,
						},
						"& .MuiTablePagination-selectLabel": {
							color: theme.palette.secondary.main,
						},
						"& .MuiInputBase-root": {
							color: theme.palette.secondary.main,
						},
						"& .MuiButton-text": {
							color: theme.palette.secondary.main,
						},
					}}
					components={{ Toolbar: GridToolbar }}
				/>
			</Box>
		</Box>
	);
}
