import {
    Box,
    Button,
    Typography,
    Modal,
    TextField,
    useTheme,
    useMediaQuery,
    TextFieldProps,
} from "@mui/material";
import {CustomTextField} from "@kitUI/CustomTextField";
import {DesktopDatePicker} from "@mui/x-date-pickers/DesktopDatePicker";
import {useEffect, useState} from "react";
import {EditPromocodeBody, Promocode} from "@root/model/promocodes";
import {enqueueSnackbar} from "notistack";


const host = window.location.hostname;
const isTest = host.includes("s");

type EditModalProps = {
    id: string;
    setId: (id: string) => void;
    promocodes: Promocode[];
    editPromocode: (body: EditPromocodeBody) => Promise<void>
};

export const EditModal = ({id, setId, promocodes, editPromocode}: EditModalProps) => {
    const theme = useTheme();
    const isMobile = useMediaQuery(theme.breakpoints.down(550));
    const promocode = promocodes.find((item) => item.id === id);
    const [descriptionField, setDescriptionField] = useState<string>("");
    const [greetingsField, setGreetingsField] = useState<string>("");
    const [dueToField, setDueToField] = useState<number>(0);
    const [activationField, setActivationField] = useState<number>(0);

    useEffect(
        function setCurrentPromocodeFields() {
            if (!promocode) return;

            setDescriptionField(promocode.description);
            setGreetingsField(promocode.greetings);
            setDueToField(promocode.dueTo);
            setActivationField(promocode.activationCount);
        },
        [promocode]
    );


    async function handleEditClick() {
        if (!promocode) return enqueueSnackbar(`Тариф ${id} не найден`);

        if (!descriptionField) return enqueueSnackbar('Поле "Описание" пустое');
        if (!greetingsField) return enqueueSnackbar('Поле "Приветственное сообщение" пустое');

        const editPromocodeBody: EditPromocodeBody = {
            id: id,
            description: descriptionField,
            greetings: greetingsField,
            dueTo: dueToField,
            activationCount: activationField,
            delete: promocode.delete
        };

       await editPromocode(editPromocodeBody);
        setId("");


    }


    return (
        <Modal
            open={Boolean(id)}
            onClose={() => {
                setId("");
            }}
            sx={{
                "& > .MuiBox-root": { outline: "none", padding: "32px 32px 16px" },
            }}
        >
            <Box
                sx={{
                    position: "absolute",
                    top: "50%",
                    left: "50%",
                    transform: "translate(-50%, -50%)",
                    width: "95%",
                    maxWidth: "600px",
                    background: "#1F2126",
                    border: "2px solid gray",
                    borderRadius: "6px",
                    boxShadow: 24,
                    p: 4,
                }}
            >
                <Box
                    sx={{
                        display: "flex",
                        gap: "30px",
                        justifyContent: "center",
                        alignItems: "center",
                        flexDirection: "column",
                    }}
                >
                    <Box
                        sx={{display: "flex",
                            flexDirection: "column",
                            gap: "15px"
                        }}
                    >
                        <Typography
                            variant="h4"
                            sx={{
                                height: "40px",
                                fontWeight: "normal",
                                color: theme.palette.secondary.main,
                            }}
                        >Редактировать промокод: {promocode?.codeword}</Typography>
                        {promocode && (
                            <>
                                <CustomTextField
                                    id={""}
                                    name="description"
                                    label="Описание"
                                    value={descriptionField}
                                    onChange={(event) => setDescriptionField(event.target.value)}
                                />
                                <CustomTextField
                                    id={""}
                                    name="greetings"
                                    label="Приветственое сообщение"
                                    value={greetingsField}
                                    onChange={(event) => setGreetingsField(event.target.value)}
                                />
                                <Typography
                                    variant="h4"
                                    sx={{
                                        height: "40px",
                                        fontWeight: "normal",
                                        marginTop: "15px",
                                        color: theme.palette.secondary.main,
                                    }}
                                >
                                    Время существования промокода
                                </Typography>
                                <DesktopDatePicker
                                    inputFormat="DD/MM/YYYY"
                                    value={dueToField === 0 ? null : new Date(Number(dueToField) * 1000)}
                                    onChange={(event: any) => {
                                        setDueToField(event.$d.getTime() / 1000 || 0);
                                    }}
                                    renderInput={(params: TextFieldProps) => <TextField {...params} />}
                                    InputProps={{
                                        sx: {
                                            height: "40px",
                                            color: theme.palette.secondary.main,
                                            border: "1px solid",
                                            borderColor: theme.palette.secondary.main,
                                            "& .MuiSvgIcon-root": { color: theme.palette.secondary.main },
                                        },
                                    }}
                                />

                                <CustomTextField
                                    id={""}
                                    name="activationCount"
                                    label="Количество активаций промокода"
                                    value={activationField}
                                    onChange={({ target }) =>
                                        setActivationField(Number(target.value.replace(/\D/g, "")))
                                    }
                                />
                            </>

                        )}

                    </Box>
                    <Box
                        sx={{
                            display: "flex",
                            gap: "30px",
                            justifyContent: "center",
                            alignItems: "center",
                        }}
                    >
                        <Button sx={{ maxWidth: "100px" }}
                                onClick={handleEditClick}
                        >
                            Сохранить изменения
                        </Button>
                        <Button sx={{ maxWidth: "100px" }} onClick={() => {
                            setId("");
                        }}>
                            Отмена
                        </Button>
                    </Box>
                </Box>
            </Box>
        </Modal>
    );
};
