import moment from "moment";
import { useEffect, useState } from "react";
import {
	Accordion,
	AccordionDetails,
	AccordionSummary,
	Table,
	TableBody,
	TableCell,
	TableHead,
	TableRow,
	Typography,
	Button,
	useTheme,
} from "@mui/material";
import { GridToolbar } from "@mui/x-data-grid";
import { enqueueSnackbar } from "notistack";
import DataGrid from "@kitUI/datagrid";

import ModalUser from "@pages/dashboard/ModalUser";
import { DateFilter } from "./DateFilter";

import { useSchildStatistics } from "@root/utils/hooks/useSchildStatistics";

import ExpandMoreIcon from "@mui/icons-material/ExpandMore";

import type { Moment } from "moment";
import type { QuizStatisticsItem } from "@root/api/quizStatistics/types";
import type { GridColDef } from "@mui/x-data-grid";

const COLUMNS: GridColDef<QuizStatisticsItem, string>[] = [
	{
		field: "user",
		headerName: "Пользователь",
		width: 250,
		valueGetter: ({ row }) => row.ID,
	},
	{
		field: "regs",
		headerName: "Регистраций",
		width: 200,
		valueGetter: ({ row }) => String(row.Regs),
	},
	{
		field: "money",
		headerName: "Деньги",
		width: 200,
		valueGetter: ({ row }) => String(row.Money),
	},
];

export const StatisticsSchild = () => {
	const [openUserModal, setOpenUserModal] = useState<boolean>(false);
	const [activeUserId, setActiveUserId] = useState<string>("");
	const [page, setPage] = useState(1);
	const [pageSize, setPageSize] = useState(10);

	const [from, setFrom] = useState<Moment | null>(moment(moment().subtract(4, "weeks")));
	const [to, setTo] = useState<Moment | null>(moment());

	const theme = useTheme();
	const statistics = useSchildStatistics(from, to).map((obj) => ({
		...obj,
		Money: Number((obj.Money / 100).toFixed(2)),
	}));

	useEffect(() => {
		if (!openUserModal) {
			setActiveUserId("");
		}
	}, [openUserModal]);

	useEffect(() => {
		if (activeUserId) {
			setOpenUserModal(true);

			return;
		}

		setOpenUserModal(false);
	}, [activeUserId]);

	const copyQuizLink = (quizId: string) => {
		navigator.clipboard.writeText(`https://${window.location.href.includes("/admin.") ? "" : "s."}hbpn.link/${quizId}`);

		enqueueSnackbar("Ссылка успешно скопирована");
	};

	return (
		<>
			<Typography sx={{ mt: "20px", mb: "20px" }}>Статистика переходов с шильдика</Typography>
			<DateFilter from={from} to={to} setFrom={setFrom} setTo={setTo} />
			<DataGrid
				sx={{ marginTop: "30px", width: "80%" }}
				getRowId={({ ID }) => ID}
				checkboxSelection={true}
				rows={statistics}
				components={{ Toolbar: GridToolbar }}
				rowCount={statistics.length}
				rowsPerPageOptions={[1, 10, 25, 50, 100]}
				paginationMode="client"
				disableSelectionOnClick
				page={page}
				pageSize={pageSize}
				onPageChange={setPage}
				onPageSizeChange={setPageSize}
				onCellClick={({ id, field }) => field === "user" && setActiveUserId(String(id))}
				getRowHeight={() => "auto"}
				columns={[
					...COLUMNS,
					{
						field: "quizes",
						headerName: "Квизы",
						flex: 1,
						minWidth: 220,
						valueGetter: ({ row }) => String(row.Quizes.length),
						renderCell: ({ row }) => (
							<Accordion sx={{ width: "100%" }}>
								<AccordionSummary
									sx={{ backgroundColor: "#26272c", color: "#FFFFFF" }}
									expandIcon={<ExpandMoreIcon sx={{ color: "#FFFFFF" }} />}
									aria-controls="panel1-content"
									id="panel1-header"
								>
									Статистика по квизам
								</AccordionSummary>
								<AccordionDetails sx={{ backgroundColor: "#26272c" }}>
									<Table
										sx={{
											width: "80%",
											border: "2px solid",
											borderColor: theme.palette.secondary.main,
											bgcolor: theme.palette.content.main,
											color: "white",
										}}
									>
										<TableHead>
											<TableRow
												sx={{
													borderBottom: "2px solid",
													borderColor: theme.palette.grayLight.main,
													height: "100px",
												}}
											>
												<TableCell sx={{ color: "inherit" }} align="center">
													QuizID
												</TableCell>
												<TableCell sx={{ color: "inherit" }} align="center">
													Регистрации
												</TableCell>
												<TableCell sx={{ color: "inherit" }} align="center">
													Деньги
												</TableCell>
											</TableRow>
										</TableHead>

										<TableBody>
											{row.Quizes.map(({ QuizID, Regs, Money }) => (
												<TableRow key={QuizID}>
													<TableCell sx={{ color: "inherit" }} align="center">
														<Button onClick={() => copyQuizLink(QuizID)}>{QuizID}</Button>
													</TableCell>
													<TableCell sx={{ color: "inherit" }} align="center">
														{Regs}
													</TableCell>
													<TableCell sx={{ color: "inherit" }} align="center">
														{(Money / 100).toFixed(2)}
													</TableCell>
												</TableRow>
											))}
										</TableBody>
									</Table>
								</AccordionDetails>
							</Accordion>
						),
					},
				]}
			/>
			<ModalUser open={openUserModal} onClose={() => setOpenUserModal(false)} userId={activeUserId} />
		</>
	);
};
