import HighlightOffOutlinedIcon from "@mui/icons-material/HighlightOffOutlined";
import SearchOutlinedIcon from "@mui/icons-material/SearchOutlined";
import { Box, Button, useMediaQuery, useTheme } from "@mui/material";
import { Ticket } from "@root/model/ticket";
import { incrementTicketsApiPage, useTicketStore } from "@root/stores/tickets";
import { useEffect, useRef, useState } from "react";
import TicketItem from "./TicketItem";
import { throttle } from "@frontend/kitui";
import makeRequest from "@root/api/makeRequest";
import { parseAxiosError } from "@root/utils/parse-error";
import { useParams } from "react-router-dom";
import CloseTicketModal from "@pages/dashboard/Content/Support/TicketList/CloseTicketModal";

type TicketListProps = {
	closeCollapse?: () => void;
	setActiveUserId: (id: string) => void;
};

export default function TicketList({ closeCollapse, setActiveUserId }: TicketListProps) {
	const theme = useTheme();
	const upMd = useMediaQuery(theme.breakpoints.up("md"));
	const tickets = useTicketStore((state) => state.tickets);
	const ticketsFetchState = useTicketStore((state) => state.ticketsFetchState);
	const ticketsBoxRef = useRef<HTMLDivElement>(null);
	const ticketId = useParams().ticketId;
	const [openModal, setOpenModal] = useState(false);

	useEffect(
		function updateCurrentPageOnScroll() {
			if (!ticketsBoxRef.current) return;

			const ticketsBox = ticketsBoxRef.current;
			const scrollHandler = () => {
				const scrollBottom = ticketsBox.scrollHeight - ticketsBox.scrollTop - ticketsBox.clientHeight;
				if (scrollBottom < ticketsBox.clientHeight && ticketsFetchState === "idle") incrementTicketsApiPage();
			};

			const throttledScrollHandler = throttle(scrollHandler, 200);
			ticketsBox.addEventListener("scroll", throttledScrollHandler);

			return () => {
				ticketsBox.removeEventListener("scroll", throttledScrollHandler);
			};
		},
		[ticketsFetchState]
	);

	const sortedTickets = tickets.sort(sortTicketsByUpdateTime).sort(sortTicketsByUnread);

	return (
		<Box
			sx={{
				display: "flex",
				flex: upMd ? "3 0 0" : undefined,
				maxWidth: upMd ? "400px" : undefined,
				maxHeight: "600px",
				flexDirection: "column",
				alignItems: "center",
			}}
		>
			<Box
				sx={{
					width: "100%",
					border: "1px solid",
					borderColor: theme.palette.grayDark.main,
					borderRadius: "3px",
					padding: "10px",
				}}
			>
				<Button
					variant="contained"
					sx={{
						backgroundColor: theme.palette.grayDark.main,
						width: "100%",
						height: "45px",
						fontSize: "15px",
						fontWeight: "normal",
						textTransform: "capitalize",
						"&:hover": {
							backgroundColor: theme.palette.menu.main,
						},
					}}
				>
					Поиск
					<SearchOutlinedIcon />
				</Button>
				<Button
					onClick={() => setOpenModal(true)}
					variant="text"
					sx={{
						width: "100%",
						height: "35px",
						fontSize: "14px",
						fontWeight: "normal",
						color: theme.palette.secondary.main,
						border: "1px solid",
						borderColor: theme.palette.golden.main,
						borderRadius: 0,
						"&:hover": {
							backgroundColor: theme.palette.menu.main,
						},
					}}
				>
					ЗАКРЫТЬ ТИКЕТ
					<HighlightOffOutlinedIcon />
				</Button>
				<CloseTicketModal openModal={openModal} setOpenModal={setOpenModal} ticketId={ticketId} />
			</Box>
			<Box
				ref={ticketsBoxRef}
				sx={{
					width: "100%",
					border: "1px solid",
					borderColor: theme.palette.grayDark.main,
					borderRadius: "3px",
					overflow: "auto",
					overflowY: "auto",
					padding: "10px",
					colorScheme: "dark",
				}}
			>
				{sortedTickets.map((ticket) => (
					<Box key={ticket.id} onClick={closeCollapse}>
						<TicketItem ticket={ticket} setActiveUserId={setActiveUserId} />
					</Box>
				))}
			</Box>
		</Box>
	);
}

function sortTicketsByUpdateTime(ticket1: Ticket, ticket2: Ticket) {
	const date1 = new Date(ticket1.updated_at).getTime();
	const date2 = new Date(ticket2.updated_at).getTime();
	return date2 - date1;
}

function sortTicketsByUnread(ticket1: Ticket, ticket2: Ticket) {
	const isUnread1 = ticket1.user === ticket1?.top_message.user_id;
	const isUnread2 = ticket2.user === ticket2?.top_message.user_id;
	return Number(isUnread2) - Number(isUnread1);
}
