import {
	Typography,
	Container,
	Button,
	Select,
	MenuItem,
	FormControl,
	InputLabel,
	useTheme,
	Box,
	TextField,
} from "@mui/material";
import { enqueueSnackbar } from "notistack";
import { requestTariffs } from "@root/services/tariffs.service";
import { createTariff } from "@root/api/tariffs";

import { findPrivilegeById, usePrivilegeStore } from "@root/stores/privilegesStore";
import { Privilege } from "@frontend/kitui";
import { currencyFormatter } from "@root/utils/currencyFormatter";
import { Formik, Field, Form, FormikHelpers } from "formik";

interface Values {
	nameField: string;
	descriptionField: string;
	amountField: string;
	customPriceField: string;
	privilegeIdField: string;
	orderField: number;
	privilege: Privilege | null;
}

export default function CreateTariff() {
	const theme = useTheme();

	const privileges = usePrivilegeStore((store) => store.privileges);

	const checkFulledFields = (values: Values) => {
		const errors = {} as any;

		if (values.nameField.length === 0) {
			errors.nameField = "Пустое название тарифа";
		}
		if (values.amountField.length === 0) {
			errors.amountField = "Пустое кол-во едениц привилегии";
		}
		if (values.privilegeIdField.length === 0) {
			errors.privilegeIdField = "Не выбрана привилегия";
		}
		return errors;
	};

	const initialValues: Values = {
		nameField: "",
		descriptionField: "",
		amountField: "",
		customPriceField: "",
		privilegeIdField: "",
		orderField: 0,
		privilege: null,
	};

	const createTariffBackend = async (values: Values, formikHelpers: FormikHelpers<Values>) => {
		if (values.privilege !== null) {
			const [, createdTariffError] = await createTariff({
				name: values.nameField,
				price: Number(values.customPriceField) * 100,
				order: Number(values.orderField),
				isCustom: false,
				description: values.descriptionField,
				privileges: [
					{
						name: values.privilege.name,
						privilegeId: values.privilege.privilegeId ?? "",
						serviceKey: values.privilege.serviceKey,
						description: values.privilege.description,
						type: values.privilege.type,
						value: values.privilege.value ?? "",
						price: values.privilege.price,
						amount: Number(values.amountField),
					},
				],
			});

			if (createdTariffError) {
				return enqueueSnackbar(createdTariffError);
			}

			enqueueSnackbar("Тариф создан");

			await requestTariffs();
		}
	};

	// const createTariffFrontend = () => {
	//   if (checkFulledFields() && privilege !== null) {
	//     updateTariffStore({
	//       id: nanoid(5),
	//       name: nameField,
	//       amount: Number(amountField),
	//       isFront: true,
	//       privilegeId: privilege.privilegeId,
	//       customPricePerUnit: customPriceField.length !== 0 ? Number(customPriceField)*100: undefined,
	//     })
	//   }
	// }

	return (
		<Formik initialValues={initialValues} validate={checkFulledFields} onSubmit={createTariffBackend}>
			{(props) => (
				<Form style={{ width: "100%" }}>
					<Container
						sx={{
							p: "20px",
							border: "1px solid rgba(224, 224, 224, 1)",
							borderRadius: "4px",
							display: "flex",
							flexDirection: "column",
							gap: "12px",
						}}
					>
						<Typography variant="h6" sx={{ textAlign: "center", mb: "16px" }}>
							Создание тарифа
						</Typography>
						<FormControl
							fullWidth
							sx={{
								height: "52px",
								color: theme.palette.secondary.main,
								"& .MuiInputLabel-outlined": {
									color: theme.palette.secondary.main,
								},
								"& .MuiInputLabel-outlined.MuiInputLabel-shrink": {
									color: theme.palette.secondary.main,
								},
							}}
						>
							<InputLabel
								id="privilege-select-label"
								sx={{
									color: theme.palette.secondary.main,
									fontSize: "16px",
									lineHeight: "19px",
								}}
							>
								Привилегия
							</InputLabel>
							<Select
								labelId="privilege-select-label"
								id="privilege-select"
								value={props.values.privilegeIdField}
								label="Привилегия"
								error={props.touched.privilegeIdField && !!props.errors.privilegeIdField}
								onChange={(e) => {
									console.log(e.target.value);
									console.log(findPrivilegeById(e.target.value));
									if (findPrivilegeById(e.target.value) === null) {
										return enqueueSnackbar("Привилегия не найдена");
									}
									props.setFieldValue("privilegeIdField", e.target.value);
									props.setFieldValue("privilege", findPrivilegeById(e.target.value));
								}}
								onBlur={props.handleBlur}
								sx={{
									color: theme.palette.secondary.main,
									borderColor: theme.palette.secondary.main,
									"&.Mui-focused .MuiOutlinedInput-notchedOutline": {
										borderColor: theme.palette.secondary.main,
										border: "1px solid",
									},
									".MuiSvgIcon-root ": {
										fill: theme.palette.secondary.main,
									},
								}}
								inputProps={{ sx: { pt: "12px" } }}
							>
								{privileges.map((privilege) => (
									<MenuItem
										data-cy={`select-option-${privilege.description}`}
										key={privilege._id}
										value={privilege._id}
										sx={{ whiteSpace: "normal", wordBreak: "break-world" }}
									>
										{privilege.serviceKey}:{privilege.description}
									</MenuItem>
								))}
							</Select>
						</FormControl>
						{props.values.privilege && (
							<Box
								sx={{
									display: "flex",
									flexDirection: "column",
								}}
							>
								<Typography>
									Имя: <span>{props.values.privilege.name}</span>
								</Typography>
								<Typography>
									Сервис: <span>{props.values.privilege.serviceKey}</span>
								</Typography>
								<Typography>
									Единица: <span>{props.values.privilege.type}</span>
								</Typography>
								<Typography>
									Стандартная цена за единицу:{" "}
									<span>{currencyFormatter.format(props.values.privilege.price / 100)}</span>
								</Typography>
							</Box>
						)}
						<Field
							as={TextField}
							id="tariff-name"
							name="nameField"
							variant="filled"
							label="Название тарифа"
							type="text"
							error={props.touched.nameField && !!props.errors.nameField}
							helperText={<Typography sx={{ fontSize: "12px", width: "200px" }}>{props.errors.nameField}</Typography>}
							InputProps={{
								style: {
									backgroundColor: theme.palette.content.main,
									color: theme.palette.secondary.main,
								},
							}}
							InputLabelProps={{
								style: {
									color: theme.palette.secondary.main,
								},
							}}
						/>
						<TextField
							id="amountField"
							name="amountField"
							variant="filled"
							onChange={(e) => {
								props.setFieldValue("amountField", e.target.value.replace(/[^\d]/g, ""));
							}}
							value={props.values.amountField}
							onBlur={props.handleBlur}
							label="Кол-во единиц привилегии"
							error={props.touched.amountField && !!props.errors.amountField}
							helperText={<Typography sx={{ fontSize: "12px", width: "200px" }}>{props.errors.amountField}</Typography>}
							InputProps={{
								style: {
									backgroundColor: theme.palette.content.main,
									color: theme.palette.secondary.main,
								},
							}}
							InputLabelProps={{
								style: {
									color: theme.palette.secondary.main,
								},
							}}
						/>
						<TextField
							id="tariff-custom-price"
							name="customPriceField"
							variant="filled"
							onChange={(e) => {
								props.setFieldValue("customPriceField", e.target.value.replace(/[^\d]/g, ""));
							}}
							value={props.values.customPriceField}
							onBlur={props.handleBlur}
							label="Кастомная цена (не обязательно)"
							InputProps={{
								style: {
									backgroundColor: theme.palette.content.main,
									color: theme.palette.secondary.main,
								},
							}}
							InputLabelProps={{
								style: {
									color: theme.palette.secondary.main,
								},
							}}
						/>
						<TextField
							id="tariff-dezcription"
							name="descriptionField"
							variant="filled"
							onChange={(e) => {
								props.setFieldValue("descriptionField", e.target.value);
							}}
							value={props.values.descriptionField}
							onBlur={props.handleBlur}
							label="Описание"
							multiline={true}
							InputProps={{
								style: {
									backgroundColor: theme.palette.content.main,
									color: theme.palette.secondary.main,
								},
							}}
							InputLabelProps={{
								style: {
									color: theme.palette.secondary.main,
								},
							}}
						/>
						<TextField
							id="tariff-order"
							name="orderField"
							variant="filled"
							onChange={(e) => {
								props.setFieldValue("orderField", e.target.value);
							}}
							value={props.values.orderField}
							onBlur={props.handleBlur}
							label="порядковый номер"
							InputProps={{
								style: {
									backgroundColor: theme.palette.content.main,
									color: theme.palette.secondary.main,
								},
							}}
							type={"number"}
							InputLabelProps={{
								style: {
									color: theme.palette.secondary.main,
								},
							}}
						/>
						<Button className="btn_createTariffBackend" type="submit" disabled={props.isSubmitting}>
							Создать
						</Button>
					</Container>
				</Form>
			)}
		</Formik>
	);
}
