import { useEffect, useRef, useState } from "react";
import { Typography, Box, useTheme, useMediaQuery } from "@mui/material";
import { DataGrid } from "@mui/x-data-grid";

import { scrollBlock } from "@root/utils/scrollBlock";

import forwardIcon from "@root/assets/icons/forward.svg";

import type { ChangeEvent } from "react";
import type { GridColDef } from "@mui/x-data-grid";

const COLUMNS: GridColDef[] = [
	{
		field: "date",
		headerName: "Дата",
		width: 130,
		sortable: false,
	},
	{
		field: "time",
		headerName: "Время",
		width: 90,
		sortable: false,
	},
	{
		field: "amount",
		headerName: "Сумма",
		width: 140,
		sortable: false,
	},
	{
		field: "email",
		headerName: "Email",
		width: 280,
		sortable: false,
	},
	{
		field: "status",
		headerName: "Статус",
		width: 100,
		sortable: false,
		renderCell: ({ value }) => (
			<Typography
				sx={{
					position: "relative",
					paddingLeft: "20px",
					fontWeight: "bold",
					fontSize: "14px",
					"&::before": {
						content: '""',
						position: "absolute",
						left: "0",
						top: "50%",
						transform: "translateY(-50%)",
						height: "13px",
						width: "13px",
						background: value ? "#0D9F00" : "#F18956",
						borderRadius: "50%",
					},
				}}
			>
				{value ? "accepted" : "timeout"}
			</Typography>
		),
	},
];

const ROWS = [
	{
		id: "row_1",
		date: "19.02.2023",
		time: "17:01",
		amount: "2 065 руб.",
		email: "emailexample@gmail.com",
		status: true,
	},
	{
		id: "row_2",
		date: "28.02.2023",
		time: "10:43",
		amount: "21 250 руб.",
		email: "verylongemailexample@gmail.com",
		status: true,
	},
	{
		id: "row_3",
		date: "04.03.2023",
		time: "21:09",
		amount: "108 065 руб.",
		email: "emailexample@gmail.com",
		status: false,
	},
];

export const TransactionsTab = () => {
	const [canScrollToRight, setCanScrollToRight] = useState<boolean>(true);
	const [canScrollToLeft, setCanScrollToLeft] = useState<boolean>(false);
	const theme = useTheme();
	const smallScreen = useMediaQuery(theme.breakpoints.down(1070));
	const gridContainer = useRef<HTMLDivElement>(null);

	useEffect(() => {
		const handleScroll = (nativeEvent: unknown) => {
			const { target } = nativeEvent as ChangeEvent<HTMLDivElement>;

			if (target.scrollLeft > 0) {
				setCanScrollToLeft(true);
			} else {
				setCanScrollToLeft(false);
			}

			if (target.clientWidth + target.scrollLeft >= target.scrollWidth - 1) {
				setCanScrollToRight(false);
			} else {
				setCanScrollToRight(true);
			}
		};

		const addScrollEvent = () => {
			const grid = gridContainer.current?.querySelector(".MuiDataGrid-virtualScroller");

			if (grid) {
				grid.addEventListener("scroll", handleScroll);

				return;
			}

			setTimeout(addScrollEvent, 100);
		};

		addScrollEvent();

		return () => {
			const grid = gridContainer.current?.querySelector(".MuiDataGrid-virtualScroller");

			grid?.removeEventListener("scroll", handleScroll);
		};
	}, []);

	const scrollDataGrid = (toStart = false) => {
		const grid = gridContainer.current?.querySelector(".MuiDataGrid-virtualScroller");

		if (grid) {
			scrollBlock(grid, { left: toStart ? 0 : grid.scrollWidth });
		}
	};

	return (
		<Box sx={{ height: "100%" }} ref={gridContainer}>
			<DataGrid
				rows={ROWS}
				columns={COLUMNS}
				pageSize={5}
				rowsPerPageOptions={[5]}
				hideFooter
				disableColumnMenu
				disableSelectionOnClick
				rowHeight={50}
				headerHeight={50}
				experimentalFeatures={{ newEditingApi: true }}
				sx={{
					borderRadius: "0",
					border: "none",
					"& .MuiDataGrid-columnHeaders": {
						padding: "0 25px",
						background: "#F2F3F7",
						borderBottom: "none",
					},
					"& .MuiDataGrid-main .MuiDataGrid-columnHeader": {
						outline: "none",
					},
					"& .MuiDataGrid-columnHeaderTitle": {
						fontWeight: "bold",
						userSelect: "none",
					},
					"& .MuiDataGrid-virtualScrollerRenderZone": {
						width: "100%",
					},
					"& .MuiDataGrid-iconSeparator": { display: "none" },
					"& .MuiDataGrid-main .MuiDataGrid-cell": {
						outline: "none",
						border: "none",
						justifyContent: "flex-start",
					},
					"& .MuiDataGrid-row": {
						padding: "0 25px",
						width: "100%",
						"&:hover": {
							background: "#F2F3F7",
						},
					},
				}}
			/>
			{smallScreen && (
				<Box
					sx={{
						position: "absolute",
						right: "15px",
						bottom: "50px",
						display: "flex",
						columnGap: "5px",
					}}
				>
					{canScrollToLeft && (
						<Box
							sx={{
								display: "flex",
								justifyContent: "center",
								alignItems: "center",
								width: "30px",
								height: "30px",
								borderRadius: "8px",
								border: "1px solid #7E2AEA",
								background: "rgba(126, 42, 234, 0.07)",
							}}
							onClick={() => scrollDataGrid(true)}
						>
							<img src={forwardIcon} alt="forward" style={{ transform: "rotate(180deg)" }} />
						</Box>
					)}
					{canScrollToRight && (
						<Box
							sx={{
								display: "flex",
								justifyContent: "center",
								alignItems: "center",
								width: "30px",
								height: "30px",
								borderRadius: "8px",
								border: "1px solid #7E2AEA",
								background: "rgba(126, 42, 234, 0.07)",
							}}
							onClick={() => scrollDataGrid(false)}
						>
							<img src={forwardIcon} alt="forward" />
						</Box>
					)}
				</Box>
			)}
		</Box>
	);
};
