import { FetchState } from "@frontend/kitui";
import { TicketMessage } from "@root/model/ticket";
import { create } from "zustand";
import { devtools } from "zustand/middleware";

interface MessageStore {
	messages: TicketMessage[];
	fetchState: "idle" | "fetching" | "all fetched";
	apiPage: number;
	messagesPerPage: number;
	lastMessageId: string | undefined;
	isPreventAutoscroll: boolean;
	ticketMessagesFetchState: FetchState;
}

export const useMessageStore = create<MessageStore>()(
	devtools(
		(set, get) => ({
			messages: [],
			messagesPerPage: 10,
			fetchState: "idle",
			apiPage: 0,
			lastMessageId: undefined,
			isPreventAutoscroll: false,
			ticketMessagesFetchState: "idle",
		}),
		{
			name: "Messages",
			enabled: process.env.NODE_ENV === "development",
		}
	)
);

export const addOrUpdateMessages = (receivedMessages: TicketMessage[]) => {
	const state = useMessageStore.getState();
	const messageIdToMessageMap: { [messageId: string]: TicketMessage } = {};

	[...state.messages, ...receivedMessages].forEach((message) => (messageIdToMessageMap[message.id] = message));

	const sortedMessages = Object.values(messageIdToMessageMap).sort(sortMessagesByTime);

	useMessageStore.setState({
		messages: sortedMessages,
		lastMessageId: sortedMessages.at(-1)?.id,
	});
};

export const clearMessageState = () =>
	useMessageStore.setState({
		messages: [],
		apiPage: 0,
		lastMessageId: undefined,
		fetchState: "idle",
	});

export const setMessageFetchState = (fetchState: MessageStore["fetchState"]) =>
	useMessageStore.setState({ fetchState });

export const incrementMessageApiPage = () => {
	const state = useMessageStore.getState();

	useMessageStore.setState({ apiPage: state.apiPage + 1 });
};

export const setIsPreventAutoscroll = (isPreventAutoscroll: boolean) =>
	useMessageStore.setState({ isPreventAutoscroll });

export const setTicketMessagesFetchState = (ticketMessagesFetchState: FetchState) =>
	useMessageStore.setState({ ticketMessagesFetchState });

function sortMessagesByTime(ticket1: TicketMessage, ticket2: TicketMessage) {
	const date1 = new Date(ticket1.created_at).getTime();
	const date2 = new Date(ticket2.created_at).getTime();
	return date1 - date2;
}
