import { create } from "zustand";
import { devtools } from "zustand/middleware";
import { Tariff } from "@frontend/kitui";
import { GridSelectionModel } from "@mui/x-data-grid";

interface TariffStore {
	tariffs: Tariff[];
	editTariffId: string | null;
	deleteTariffIds: string[] | null;
	selectedTariffIds: GridSelectionModel;
}

export const useTariffStore = create<TariffStore>()(
	devtools(
		(set, get) => ({
			tariffs: [],
			editTariffId: null,
			deleteTariffIds: null,
			selectedTariffIds: [],
		}),
		{
			name: "Tariffs",
			enabled: process.env.NODE_ENV !== "production",
		}
	)
);

export const updateTariffs = (tariffs: TariffStore["tariffs"], mode: "add" | "replace" = "replace") =>
	useTariffStore.setState(
		(state) => {
			const tariffMap: Record<string, Tariff> = {};

			[...(mode === "add" ? state.tariffs : []), ...tariffs].forEach((tariff) => (tariffMap[tariff._id] = tariff));

			const sortedTariffs = Object.values(tariffMap).sort(sortTariffsByCreatedAt);

			return { tariffs: sortedTariffs };
		},
		false,
		{
			type: "updateTariffs",
			tariffsLength: tariffs.length,
		}
	);

export const openEditTariffDialog = (editTariffId: TariffStore["editTariffId"]) =>
	useTariffStore.setState({ editTariffId });

export const closeEditTariffDialog = () => useTariffStore.setState({ editTariffId: null });

export const openDeleteTariffDialog = (deleteTariffId: TariffStore["deleteTariffIds"]) =>
	useTariffStore.setState({ deleteTariffIds: deleteTariffId });

export const closeDeleteTariffDialog = () => useTariffStore.setState({ deleteTariffIds: null });

export const setSelectedTariffIds = (selectedTariffIds: TariffStore["selectedTariffIds"]) =>
	useTariffStore.setState({ selectedTariffIds });

function sortTariffsByCreatedAt(tariff1: Tariff, tariff2: Tariff) {
	if (!tariff1.createdAt || !tariff2.createdAt) throw new Error("Trying to sort tariffs without createdAt field");

	const date1 = new Date(tariff1.createdAt).getTime();
	const date2 = new Date(tariff2.createdAt).getTime();
	return date1 - date2;
}
